#include <afxwin.h>
#include <afxext.h>
#include "resource.h"
#include "docview.h"

MyApp App; // instantiate the application

IMPLEMENT_DYNCREATE(MyWindow, CFrameWnd)
IMPLEMENT_DYNCREATE(MyView, CView)
IMPLEMENT_DYNCREATE(MyDocument, CDocument)

// This is the application's message map.
BEGIN_MESSAGE_MAP(MyApp, CWinApp)
  ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
  ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
  ON_COMMAND(ID_FILE_PRINT_SETUP,CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

// This is the view's message map.
BEGIN_MESSAGE_MAP(MyView, CView)
  ON_WM_LBUTTONDOWN()
  ON_COMMAND(ID_SHAPES_SQUARE,OnSquare)
  ON_COMMAND(ID_SHAPES_CIRCLE,OnCircle)
  ON_COMMAND(ID_SHAPES_PICTURE,OnPicture)
  ON_COMMAND(ID_FILE_PRINT,CView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_PREVIEW,CView::OnFilePrintPreview)
END_MESSAGE_MAP()

// This is the document's message map.
BEGIN_MESSAGE_MAP(MyDocument, CDocument)
  ON_COMMAND(IDM_UNDO, OnUndo)
  ON_COMMAND(IDM_RESET, OnReset)
  ON_COMMAND(IDM_HELP, OnHelp)
END_MESSAGE_MAP()

// This is the frame window's message map.
BEGIN_MESSAGE_MAP(MyWindow, CFrameWnd)
  // not used by this example
END_MESSAGE_MAP()

// Initialize the main window.
MyWindow::MyWindow()
{
  // Not used by this example.
}

// Initialize the application.
BOOL MyApp::InitInstance()
{
  CSingleDocTemplate *DocPtr = new CSingleDocTemplate(
    IDR_FRAMEWIN,
    RUNTIME_CLASS (MyDocument),
    RUNTIME_CLASS(MyWindow),
    RUNTIME_CLASS(MyView)
  );

  AddDocTemplate(DocPtr);

  // register application in registry
  EnableShellOpen();
  RegisterShellFileTypes();

  CCommandLineInfo CLInfo;
  ParseCommandLine(CLInfo);

  if(!ProcessShellCommand(CLInfo)) return FALSE;

  return TRUE;
}

// Initialize the document.
MyDocument::MyDocument()
{
  m_LocIndex = 0;
}

// Process IDM_HELP -- this is only placeholder help.
afx_msg void MyDocument::OnHelp()
{
   App.m_pMainWnd->MessageBox("Document/View Demo",
                              "Placeholder Help");
}

// Handle new document request.
BOOL MyDocument::OnNewDocument()
{
  if(!CDocument::OnNewDocument()) return FALSE;

  m_LocIndex = 0;

  return TRUE;
}

// Load or store document.
void MyDocument::Serialize(CArchive &arch)
{
  int i;

  if(arch.IsLoading())
  {
    arch >> m_LocIndex;
    for(i=0; i<m_LocIndex; i++)
	{
		arch >> m_Locations[i];
		arch >> m_Type[i];
	}
  }
  else
  {
    arch << m_LocIndex;
    for(i=0; i<m_LocIndex; i++)
	{
		arch << m_Locations[i];
		arch << m_Type[i];
	}
  }
}

// Undo last line.
afx_msg void MyDocument::OnUndo()
{
  m_LocIndex--;
  if(m_LocIndex<0) m_LocIndex = 0;

  SetModifiedFlag();
  UpdateAllViews(NULL);
}

// Reset point index and erase window.
afx_msg void MyDocument::OnReset()
{
  m_LocIndex = 0;

  SetModifiedFlag();
  UpdateAllViews(NULL);
}

// Record position of mouse click.
afx_msg void MyView::OnLButtonDown(UINT flags, CPoint loc)
{
  MyDocument *DocPtr = (MyDocument *) GetDocument();

  if(DocPtr->m_LocIndex == MAXLOC)
  {
    DocPtr->m_LocIndex = MAXLOC;
    App.m_pMainWnd->MessageBox("Too Many Points", "Error");
	return;
  }

  DocPtr->m_Locations[DocPtr->m_LocIndex] = loc;
  DocPtr->m_Type[DocPtr->m_LocIndex]=choice;
  DocPtr->m_LocIndex++;
  

  DocPtr->SetModifiedFlag();
  DocPtr->UpdateAllViews(NULL);
}

// Draw lines.
void MyView::OnDraw(CDC *DC)
{
  MyDocument	*DocPtr = (MyDocument *) GetDocument();
  int			i;

  for(i=0; i<DocPtr->m_LocIndex; i++)
  {
	  switch(DocPtr->m_Type[i])
	  {
	  case 1:
		  DC->Rectangle(DocPtr->m_Locations[i].x-10,
			            DocPtr->m_Locations[i].y-10,
						DocPtr->m_Locations[i].x+10,
						DocPtr->m_Locations[i].y+10);
		  break;

	  case 2:
		  DC->Ellipse(DocPtr->m_Locations[i].x-10,
			          DocPtr->m_Locations[i].y-10,
					  DocPtr->m_Locations[i].x+10,
					  DocPtr->m_Locations[i].y+10);
		  break;

	  case 3:
		  CDC memDC;
		  memDC.CreateCompatibleDC(DC);
		  memDC.SelectObject(cb);
		  DC->BitBlt(DocPtr->m_Locations[i].x-10,
			         DocPtr->m_Locations[i].y-10,
					 20,20,&memDC,0,0,SRCCOPY);
		  break;
	  }
  }
}


MyView::MyView()
{
	choice=1;
	cb.LoadBitmap(IDB_BITMAP1);
}

void MyView::OnSquare()
{
	choice=1;
}

void MyView::OnCircle()
{
	choice=2;
}

void MyView::OnPicture()
{
	choice=3;
}


/*-- PRINTING FUNCTIONS --*/
BOOL MyView::OnPreparePrinting(CPrintInfo *pInfo)
{
	pInfo->SetMaxPage(3);
	return DoPreparePrinting(pInfo);
}

void MyView::OnBeginPrinting(CDC *pDC,CPrintInfo *pInfo)
{
	// TODO: add extra initialization before printing
}

void MyView::OnEndPrinting(CDC *pDC,CPrintInfo *pInfo)
{
	// TODO: add cleanup after printing
}

void MyView::OnPrint(CDC *pDC,CPrintInfo *pInfo)
{
	CView::OnPrint(pDC, pInfo);
	char str[80];
	CFont font,*def_font;

	wsprintf(str,"Page %02d of %02d",pInfo->m_nCurPage,pInfo->GetMaxPage());
	pDC->SetMapMode(MM_LOMETRIC);
	font.CreatePointFont(240, "Arial", pDC);
	def_font = pDC->SelectObject(&font);
	pDC->TextOut(1200,-25,str);



	pDC->SetMapMode(MM_LOENGLISH);
	pDC->SelectObject(def_font);
	font.DeleteObject();
	
	CView::OnPrint(pDC,pInfo);
}
